require 'redis'
require 'json'
require 'uuid'


class SessionManager::Handler 
  
  USERS_SESSIONS = 'users_sessions'
  APPS_SESSIONS  = 'applications_sessions'


  def self.create_session(attribute,option) 
    if validate_option(option) && valid_token?(attribute,option) 
      param = parse_json(attribute)
      collection = collection(option)
      token = generate_token
      session = session_generator(option,param,token)
      result = get_redis.sadd(collection,session.to_json)
      result ? return_response(200,token) : return_response(400,"Not created")
    else
      return_response(400,"Not created")
    end
  end
 
  def self.verify_session(object,option)
   if validate_option(option) && validate_json(object,option)   
    collection = collection(option)
    result     = get_redis.sismember(collection,object)
    result ? return_response(200,"Ok") : return_response(400,"Not ok") 
   else
       return_response(400,"Ok")
   end
 end

 
 def self.delete_session(object,option)
    if validate_option(option) && validate_json(object,option)
     collection = collection(option)
     result     = get_redis.srem(collection,object)
     result ? return_response(200,"Succesfully deleted") : return_response(400,"Not deleted")
    else
        return_response(400,"Not deleted")
    end
 end

 private #######################################################################

 def self.get_redis
     redis = Redis.current
     redis
 end


 def self.get_uuid
     uuid = UUID.new
     uuid
 end

#generate token
def self.generate_token
    get_uuid.generate
end

 #determine the database collection to use
 def self.collection(option)
     user = option.fetch(:user) || false
     app  = option.fetch(:app)  || false
     collection = user ? USERS_SESSIONS : APPS_SESSIONS
     return collection
 end

 #parse json
 def self.parse_json(param)
   JSON.parse(param)
 end

 #create a session hash for the database
 def self.session_generator(option, param, token)
   option[:user] ? { login: param['login'],token: token} : { api_key: param['api_key'],token: token} 
 end

 #check if the passed keys are those which are waited
 def self.valid_keys?(param,option)
   option[:user] ? waited_keys = ["login","token"] : waited_keys = ["api_key","token"] 
   waited_keys.each do |k|
      if !(param.has_key?(k))
        return false
        break
      end
    end
    return true
 end

 #check if the passed token is valid
 def self.valid_token?(token,option)
   option[:user] ? attr = "login" : attr = "api_key"
   begin
     object = parse_json(token)
     hash   = object.to_hash
     if !(hash.has_key?(attr)) || !(hash[attr].is_a?(String))
       return false
     end
   rescue
     return false
   end
 return true
 end

 #check if hash's values are String
 def self.validate_values(hash)
   values = hash.values
   values.each do |v|
     if !(v.is_a?(String))
       return false
       break
     end
   end
   return true
 end

 #check if the hash values are not nil neither empty
 def self.validate_argument(param)
   param.values.each do |v|
    if v.nil? || v.empty?
      return false
      break
    end
   end
   return true
 end
 
 #check if the json object is valid
 def self.validate_json(param,option)
   begin
    object  = parse_json(param) 
    hash    = object.to_hash
    return valid_keys?(hash,option) && validate_argument(hash) && validate_values(hash)
   rescue
     return false
   end
 end

 #check if the option is valid
 def self.validate_option(option)
  if !(option.nil?) && option.is_a?(Hash) && !(option.empty?) && option.has_key?(:user) && option.has_key?(:app)
    if option[:user] && option[:app]
      return false
    else 
      if !option[:user] && !option[:app]
       return  false
      else
        option.values.each do |v|
          unless (v.is_a?(TrueClass) || v.is_a?(FalseClass))
            return false
            break
          end
        end
      end
    end
  else
    return false
  end
  return true
end

#change the return parameter in hash format
def self.return_response(code,msg)
  {code: code,message: msg}
end


end
